/// @file   Socket.cpp
///

#include <Socket.hpp>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <arpa/inet.h>

#include <strings.h>
#include <spdlog/spdlog.h>

Socket::Socket(int fd)
    : m_fd{fd}
{

}


Socket::Socket(Socket&& move)
    : m_fd(-1)
{
    spdlog::debug("Socket is now unvalid.");
    move.swap(*this);
}

Socket& Socket::operator=(Socket&& move)
{
    move.swap(*this);
    return *this;
}

void Socket::swap(Socket& other)
{
    using std::swap;
    swap(m_fd,   other.m_fd);
}



Socket::~Socket()
{
    if(m_fd != -1)
    {
        close();
    }
}

int Socket::get_socket() const
{
    return m_fd;
}

void Socket::close()
{
    while (true)
    {
        int state = ::close(m_fd);
        if (state == -1)
        {
            break;
        }
        else
        {
            switch (errno)
            {
            case EIO:
                break;
            
            default:
                break;
            }
        }
    }
    spdlog::debug("socket closed");
    m_fd = -1;
}

DataSocket::DataSocket(int fd)
    : Socket(fd)
{

}

ConnectSocket::ConnectSocket(std::string const& host, int port)
    : DataSocket(::socket(PF_INET, SOCK_STREAM, 0))
{
    struct sockaddr_in serverAddr {};
    serverAddr.sin_family = AF_INET;
    serverAddr.sin_port = htons(port);
    serverAddr.sin_addr.s_addr = inet_addr(host.c_str());

    if(::connect(get_socket(), (struct sockaddr*)&serverAddr, sizeof(serverAddr)) != 0)
    {
        close();
    }
}

ServerSocket::ServerSocket(int port)
    : Socket(::socket(PF_INET, SOCK_STREAM, 0))
{
    struct sockaddr_in serverAddr;
    bzero((char*)&serverAddr, sizeof(serverAddr));
    serverAddr.sin_family = AF_INET;
    serverAddr.sin_port = htons(port);
    serverAddr.sin_addr.s_addr = INADDR_ANY;

    if (::bind(get_socket(), (struct sockaddr *) &serverAddr, sizeof(serverAddr)) != 0)
    {
        close();
    }

    if (::listen(get_socket(), max_connections) != 0)
    {
        close();
    }
}

DataSocket ServerSocket::accept()
{
    struct sockaddr_storage serverStorage;
    socklen_t addr_size = sizeof(serverStorage);
    int new_socket = ::accept(get_socket(), (struct sockaddr*)&serverStorage, &addr_size);
    if (new_socket == -1)
    {
        spdlog::error("Failed to accept connection, new fd socket not valid for DataSocket");
        spdlog::debug("errno <{}>", errno);
    }
    return DataSocket(new_socket);
}


/// @brief Write data into socket, unitl everything is written
///         or an error occured.
///
void DataSocket::put_data(char const* data, std::size_t size)
{
    std::size_t data_written {0};
    while (data_written < size)
    {
        auto put = write(get_socket(), data + data_written, size - data_written);
        if (put != -1)
        {
            data_written += put;
        }
        else
        {
            break;
        }
    }
}

std::size_t DataSocket::get_data(char* data, std::size_t size)
{
    std::size_t data_read {0};

    while (data_read < size)
    {
        auto get = read(get_socket(), data + data_read, size - data_read);
        if (get != -1)
        {
            data_read += get;
            if (get == 0) {break;}
        }
        else
        {
            spdlog::debug("Socket read returned -1");

            switch (errno)
            {
            case ENXIO:
                spdlog::error("ENXIO: Fatal programming bug");
                break;
            case ENOMEM:
                spdlog::error("ENOMEM: Ressource acquisition failuire");
                break;
            case ENOTCONN:
                spdlog::debug("ENOTCONN: Connection is broken.");
                get = 0;
                break;
            case EAGAIN:
                spdlog::debug("EAGAIN: Trying again.");
                continue;
            case EFAULT:
                spdlog::debug("EFAULT: Bad address, fd<{}>", get_socket());
                break;
            default:
                spdlog::error("Unknown error with id <{}> occured. Check errno-base.h for more information.", errno);
                break;
            }
            break;
        }
    }
    return data_read;
}


void DataSocket::end_transmission()
{
    if (::shutdown(get_socket(), SHUT_WR) != 0)
    {
        spdlog::error("shutting down transmission was not successfull.");
    }
}