#include "Node.hpp"

#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/i2c-dev.h>
#include <unistd.h>

// #include <spdlog/spdlog.h>

using namespace i2c;

Node::Node(uint8_t node_address)
    : m_address{node_address}, m_device{-1}
{
}

Node::~Node()
{
    close(m_device);
}

bool Node::open_device(const char *i2c_device_name)
{
    auto fd = open(i2c_device_name, O_RDWR);
    if (fd < 0)
    {
        // spdlog::error("Failed to open device {}", i2c_device_name);
        return false;
    }
    auto success_selecting_device = ioctl(fd, I2C_SLAVE, m_address);
    if (success_selecting_device < 0)
    {
        // spdlog::error("Failed to select i2c node {}", m_address);
        return false;
    }
    m_device = fd;
    return true;
}

bool Node::send(uint8_t *data, ssize_t size)
{
    ssize_t bytes_written = write(m_device, data, size);
    if (bytes_written != size)
    {
        // spdlog::error("Error while writing bytes. Written {} bytes instead of {}!", bytes_written, size);
        return false;
    }
    return true;
}

bool Node::send(std::vector<uint8_t> &data)
{
    return send(data.data(), data.size());
}

bool Node::send(std::array<uint8_t, 2> &data)
{
    return send(data.data(), data.size());
}

bool Node::send(uint8_t reg, uint8_t val)
{
    uint8_t data[2]{reg, val};
    return send(data, 2);
}

constexpr uint8_t i2c_smbus_max_block_size = 32;
struct i2c_data
{
    uint8_t block[i2c_smbus_max_block_size + 2];
};

struct i2c_ioctl_data
{
    char read_write;
    uint8_t command;
    int size;
    i2c_data *data;
};

int Node::read16(uint8_t reg_addr)
{
    i2c_data data;
    i2c_ioctl_data args;
    args.read_write = 1; // rw
    args.command = reg_addr;
    args.size = 3; // byte = 2; word = 3 (because of address); only byte: 1 -> command = 0;
    args.data = &data;
    auto success = ioctl(m_device, I2C_SMBUS, &args);
    if (success != 0)
    {
        // error
        // spdlog::error("Error while reading data.");
        return -1;
    }
    else
    {
        int answer{0};
        answer |= static_cast<int>(data.block[0] << 8); // MSB
        answer |= static_cast<int>(data.block[1] << 0); // LSB
        // spdlog::debug("Received data: {} {} converted to {}.", data.block[0], data.block[1], answer);
        return answer;
    }
}