/// @file   Node.hpp
/// @brief  I2c node class.
///
/// @details    Use the GPIO pins with the wiring pi library
///             for i2c communication.
///

#ifndef I2C_NODE_HEADER
#define I2C_NODE_HEADER

#include <stdint.h>
#include <array>
#include <vector>

namespace i2c
{

/// @brief Node class
///
class Node
{
public:
    /// @brief Create node device
    ///
    /// @param node_address Address to talk to.
    ///
    Node(uint8_t node_address);
    ~Node();

    /// @brief Open i2c device
    /// 
    /// @param device_name Name of the i2c device to use: e.g. "/dev/i2c-1"
    /// @return Returns wether opening of the device was successfull.
    ///
    bool open_device(const char* i2c_device_name);


    /// @brief Write vector of values to node
    ///
    /// @param data Data that should be written
    /// @return If write operation was successfull
    //
    bool send(uint8_t* data, ssize_t size);
    bool send(std::vector<uint8_t> & data);
    bool send(std::array<uint8_t, 2> &data);
    bool send(uint8_t reg, uint8_t val);
    
    /// @brief Read from node
    ///
    /// @param  regAddress Address of the register to 
    ///         read from
    /// @return Returns the received value from the register
    ///
    int read16(uint8_t reg_addr);

private:
    uint8_t m_address; ///< Adress of the node.
    int m_device; ///< File descriptor of i2c device.
};

} // i2c namespace


#endif