/// @file HardwareControl.cpp
///

#include <HardwareControl.hpp>
#include <spdlog/spdlog.h>

#include <thread>

HardwareControl::HardwareControl(i2c::Node& node, DefaultState initial_state)
    : m_modell{node}
{
    put_modell_into_state(initial_state);
}

HardwareControl::~HardwareControl()
{
    put_modell_into_state(DefaultState::Off);
}


void HardwareControl::put_modell_into_state(DefaultState state) const
{
    switch (state)
    {
    case DefaultState::Off:
        default_state_off();
        break;
    
    case DefaultState::Fancy:
        default_state_fancy();
    default:
        break;
    }
}


void HardwareControl::default_state_off() const
{
    update_windmill_speed(0);
    set_village_color(0, 0);
    set_power_plant(0);
    set_solar_plant(0);
    set_windmill_net(0);
    set_renewable_net(0);
}


void HardwareControl::default_state_fancy() const
{
    update_windmill_speed(50);
    set_village_color(20, 20);
    set_power_plant(20);
    set_solar_plant(20);
    set_windmill_net(20);
    set_renewable_net(20);
}


void HardwareControl::update_windmill_speed(uint8_t speed) const
{
    constexpr uint8_t min_motor_speed {40};
    if (speed == 0)
    {
        set_output_pin(Pin::WindmillMotor, 0);
    }
    else if (speed > 180)
    {
        set_output_pin(Pin::WindmillMotor, 140);
    }
    else if (speed <= min_motor_speed)
    {
        set_output_pin(Pin::WindmillMotor, min_motor_speed);
    }
    else
    {
        set_output_pin(Pin::WindmillMotor, speed);
    }
}


void HardwareControl::set_village_color(uint8_t red, uint8_t green) const
{
    set_output_pin(Pin::VillageNetRed, red);
    set_output_pin(Pin::VillageNetGreen, green);
    if (green < 200)
    {
        green = 0; // green can only be on or off;
    }

    for (auto house : {House::One, House::Two, House::Three, House::Four, House::Five})
    {
        set_house_color(house, red, green);
    }
}

void HardwareControl::set_power_plant(uint8_t brightness) const
{
    set_output_pin(Pin::PowerPlantNet, brightness);
}

void HardwareControl::set_solar_plant(uint8_t brightness) const
{
    set_output_pin(Pin::SolarFieldNet, brightness);
}

void HardwareControl::set_windmill_net(uint8_t brightness) const
{
    set_output_pin(Pin::WindmillNet, brightness);
}

void HardwareControl::set_renewable_net(uint8_t brightness) const
{
    set_output_pin(Pin::WindAndSolarNet, brightness);
}


void HardwareControl::set_output_pin(Pin pin, uint8_t value) const
{
    bool succ = m_modell.send(static_cast<uint8_t>(pin), value);
    spdlog::debug("Send: {} {}, success: {}", pin, value, succ);
    using namespace std;
    this_thread::sleep_for(chrono::milliseconds(14));
}

void HardwareControl::set_house_color(House number, uint8_t red, uint8_t green) const
{
    switch (number)
    {
    case House::One:
        set_output_pin(Pin::House1Red, red);
        set_output_pin(Pin::House1Green, green);
        break;
    case House::Two:
        set_output_pin(Pin::House2Red, red);
        set_output_pin(Pin::House2Green, green);
        break;
    case House::Three:
        set_output_pin(Pin::House3Red, red);
        set_output_pin(Pin::House3Green, green);
        break;
    case House::Four:
        set_output_pin(Pin::House4Red, red);
        set_output_pin(Pin::House4Green, green);
        break;
    case House::Five:
        set_output_pin(Pin::House5Red, red);
        set_output_pin(Pin::House5Green, green);
        break;
    
    default:
        break;
    }
}