

#ifndef HARDWARE_CONTROL_HEADER
#define HARDWARE_CONTROL_HEADER


#include <i2c/Node.hpp>

class HardwareControl
{
public:

    /// @brief Pre defined states of the smart grid modell.
    ///
    enum class DefaultState
    {
        Off,    ///< just turn everything off.
        Fancy   ///< show some fancy lights.
    };

    /// @biref Enumeration of houses on the smart grid modell.
    ///
    enum class House
    {
        One,
        Two,
        Three,
        Four,
        Five
    };

    /// @brief Pins that are used on the arduino
    ///
    enum class Pin : uint8_t
    {
        WindmillMotor   = 0x02,    ///< 2
        Sun             = 0x03,    ///< 3
        SolarFieldNet   = 0x04,    ///< 4
        WindmillNet     = 0x05,    ///< 5
        WindAndSolarNet = 0x06,    ///< 6
        PowerPlantNet   = 0x07,    ///< 7
        VillageNetGreen = 0x08,    ///< 8
        VillageNetRed   = 0x09,    ///< 9

        House1Red       = 0x2f,    ///< 47
        House1Green     = 0x29,    ///< 41
        House2Red       = 0x35,    ///< 53
        House2Green     = 0x2d,    ///< 45
        House3Red       = 0x33,    ///< 51
        House3Green     = 0x31,    ///< 49
        House4Red       = 0x2B,    ///< 43
        House4Green     = 0x25,    ///< 37
        House5Red       = 0x27,    ///< 39
        House5Green     = 0x23     ///< 35
    };

    /// @brief Initialiese modell with everything off.
    ///
    HardwareControl(i2c::Node& node, DefaultState initial_state = DefaultState::Off);

    ~HardwareControl();


    /// @brief Put the to one of the predefined states.
    /// 
    /// @param state Default state to which the modell will be changed.
    ///
    void put_modell_into_state(DefaultState state) const;

    /// @brief Set the turning speed of the windmill.
    /// @details 
    /// Checks if the value is high enough to turn the mill.
    /// If not high enough the motor will be turned off.
    ///
    /// @param speed Turning speed of the windmill.
    ///
    void update_windmill_speed(uint8_t speed) const;
    
    /// @brief Set the color mix of the whole village.
    ///
    /// @param red Brightness of the red LEDs.
    /// @param green Brightness of the green LEDs.
    //
    void set_village_color(uint8_t red, uint8_t green) const;
    
    /// @brief Set brightness of the power plant (only red LEDs)
    ///
    /// @param brightness Brightness of the power plant LEDs.
    ///
    void set_power_plant(uint8_t brightness) const;
    
    /// @brief Set brightness of the solar plant LEDs.
    /// 
    /// @param brightness Brightness of the solar power plant LEDs.
    ///
    void set_solar_plant(uint8_t brightness) const;
    
    /// @brief Set brightness of the windmill LEDs.
    ///
    /// @param brightness Brightness of the windmill LEDs.
    ///    
    void set_windmill_net(uint8_t brightness) const;
    
    /// @brief Set the brightness of the combined renewable
    ///        energy net.
    ///
    /// @param brightness Brightness off the LEDs.
    ///
    void set_renewable_net(uint8_t brightness) const;

private:

    /// @brief Set the color of the given house.
    ///
    /// @param number Selected house.
    /// @param red Brightness of red LEDs.
    /// @param green Brightness of green LEDs.
    ///
    void set_house_color(House number, uint8_t red, uint8_t green) const;
    
    void default_state_off() const;
    void default_state_fancy() const;

    
    void set_output_pin(Pin pin, uint8_t value) const;

    i2c::Node& m_modell;     ///< I2C connection to the Arduino

};

#endif